console.log("3D.js");

// DEBUG CONFIG //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
isAndroid = false;

// CONFIG 3D ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
var config3D   = {};
config3D.visor = {
    ancho: 1024,
    alto : 517 
};
config3D.visor.ratioXY = config3D.visor.ancho / config3D.visor.alto;
config3D.camara = {
    frustumVertical: 45,
    nearPlane: 1,
    farPlane: 1000
};

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
var renderer; 
var webGLsupported    = true;
var estoyHaciendoZoom = false;
var estoyMoviendome   = false;
var cuandoSolte       = false;
var lastPDPx          = false;
var lastPDPy          = false;

try {
    renderer = new THREE.WebGLRenderer({
        antialias: true,
        alpha: true
    });
} catch(error) {
    webGLsupported = false;
}

var scene   = new THREE.Scene();
var camera  = new THREE.PerspectiveCamera( config3D.camara.frustumVertical, config3D.visor.ratio, config3D.camara.nearPlane, config3D.camara.farPlane );
var ambient = new THREE.AmbientLight(0xFFFFFF);

camera.aspect = ( config3D.visor.ratioXY );

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

var inTouchObject = false;
var gottenObject  = false;
// var bindedObjects = [];

var tipoInteraccion;
// var herramientaSeleccionada = "rotar";
var herramientaSeleccionada = false;
var showMenu = false;
var baseCameraParent;
var dondeMiro;
var objetoCapa = [];
var d = 1;
var arrayMenuSeleccinado = [];
var itemMenuSeleccionado = false;
var interCapas;
var controles;
var materialesGuardados = {};
var distance = 30;
var basePosition   = {};
var baseReferencia = {};
var baseCamera = {};

/// NRD functions /////////////////////////////////////////////////////////
function initNRD() {
    debug.log( "OK..." );
    document.title = "Santillana";
    $("body").addClass( nrd.config.nrdJson.style );
    loadNRD();
}

function loadNRD() {
    ims.interfaz.init();
    $( "#headerText" ).append( nrd.config.nrdJson.title );
    if ( !! webGLsupported ){
        var tdContainer = '<div id="3dContainer" class="noShow"></div>';
        $( "#visor" ).append( tdContainer );
        $( "#3dContainer" ).append( renderer.domElement );
        loadResource();
    }
    else {
        loadErrorWebGL();
    }
}

function loadErrorWebGL() {
    removeLoaderPlaceholder();
    $( "#visor" ).append( '<div id="errorWebGL"><h1>Hay un problema con la aceleraci&oacute;n 3D</h1><p>Lo sentimos, pero hay un problema con las opciones 3D de su dispositivo. Puede consultar la informaci&oacute;n disponible en la siguiente pagina web <a href="https://get.webgl.org/">WebGL</a>.</p></div>' );
}

// HELPER 3D ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function loadResource() {
    var _loader = '<div class="sk-spinner sk-spinner-wave" id="loaderWEBGL"><div class="sk-rect1"></div><div class="sk-rect2"></div><div class="sk-rect3"></div><div class="sk-rect4"></div>'+
      '<div class="sk-rect5"></div></div>';
    $("#visor").prepend( _loader );

    var loader = new THREE.ColladaLoader();
    var url    = nrd.config.url + '/recurso.dae';

    loader.load(url, function(obj) {
        // console.clear();
        scene.add( obj.scene );
        loadCapas();
        removeLoaderPlaceholder();
        $("#3dContainer").removeClass( "noShow" );
        requestAnimationFrame( doRender );
    }, function(xhr) {
        var _percent = xhr.loaded / xhr.total * 100;
        if ( _percent > 15 ){
            $(".sk-rect1").css({
                "background-color": "fuchsia"
            });
        }
        if ( _percent > 35 ){
            $(".sk-rect2").css({
                "background-color": "fuchsia"
            });
        }
        if ( _percent > 55 ){
            $(".sk-rect3").css({
                "background-color": "fuchsia"
            });
        }
        if ( _percent > 75 ){
            $(".sk-rect4").css({
                "background-color": "fuchsia"
            });
        }
        if ( _percent > 95 ){
            $(".sk-rect5").css({
                "background-color": "fuchsia"
            });
        } 
    });
}

var lastGap = 0;
function setControls() {
    var cameraparent;
    var referencia;
    var tapTimeTouch;
    var posicion  = new THREE.Vector3();
    var cameraAng = new THREE.Vector3(0, 30, 0);
    var touchOn = false;
    var touchesDistance = 0;
    var zoomVar = 0;
    var onPointerDownPointerX = 0;
    var onPointerDownPointerY = 0;

    var unDedo = false;
    var unDedoStart = false;

    var angX  = 0;
    var angY  = 2.09;
    var moveX = 0;
    var moveY = 0;
    var desplazamiento   = {};
    var renderClearColor = parseInt(nrd.config.nrdJson.background) || 0xEEEEEE;

    //var objetoReferencia     = scene.getObjectByName('referencia'       , true) || false;
    var objetoRecurso        = scene.getObjectByName('recurso'    , true) || false;
    var objetoSphere         = scene.getObjectByName('sphere'     , true) || false;
    var objetoStartingCamara = scene.getObjectByName('main_camera', true) || false;
    var lights               = scene.getObjectByName('lights'     , true) || false;

    // Setting up lights
    if (lights.children) {
       var light            = new THREE.DirectionalLight(0xEEEEEE, 0.35);
       light.position       = lights.children[0].position;
       light.shadowBias     = 0.15;
       light.castShadow     = true;
       light.shadowDarkness = 0.6;
       scene.add(light);
    }

    renderer.setClearColor(renderClearColor, 1);
    renderer.setSize(config3D.visor.ancho, config3D.visor.alto);
    
    renderer.shadowMapEnabled  = true;
    
    scene.traverse(function(obj) {
        if (obj instanceof THREE.Mesh) {
            obj.receiveShadow = true;
            obj.castShadow    = true;
        }
    });

    basePosition.position = objetoSphere.position.clone();
    basePosition.rotation = objetoSphere.rotation.clone();
    baseCamera.position   = objetoStartingCamara.position.clone();

    camera.position.set( objetoStartingCamara.position.x, objetoStartingCamara.position.y, objetoStartingCamara.position.z );
    camera.lookAt( objetoSphere.position );
    camera.updateProjectionMatrix();

    scene.add(camera);
    scene.add(ambient);

    if (!isMobile) {
        function mousedown(event) {
            if (!herramientaSeleccionada) {
                return;
            }

            if(!touchOn) {
                touchOn = true;
                onPointerDownPointerX = event.clientX;
                onPointerDownPointerY = event.clientY;
            }
        }

        function mousemove(event) {
            if (!herramientaSeleccionada) {
                return;
            }

            if(touchOn) {
                moveX = (event.clientX - onPointerDownPointerX) / config3D.visor.ancho * 45;
                moveY = (event.clientY - onPointerDownPointerY) / config3D.visor.alto  * 45;
                switch ( herramientaSeleccionada ) {
                    case "mover":
                        var x = moveX * 0.05 * objetoSphere.position.distanceTo(camera.position);
                        var y = moveY * 0.05 * objetoSphere.position.distanceTo(camera.position);
                        break;
                    case "rotar":
                        var x = moveX * 0.08;
                        var y = moveY * 0.08;
                        break;
                }
                onPointerDownPointerX = event.clientX;
                onPointerDownPointerY = event.clientY;
                moveCamera(x, y);
            }
        }

        function mouseup(event) {  
            if (!herramientaSeleccionada) {
                return;
            }
            touchOn = false;
        }

        function mousewheel(event) {
            if (!event.wheelDeltaY && !event.wheelDelta && !event.detail){
                event = event.originalEvent;
            }
            // WebKit | Opera | IE
            if (event.wheelDeltaY) {
                moveCameraDo(-event.wheelDeltaY / 50);
            } else if (event.detail) { // Firefox
                moveCameraDo(event.detail);
            }
        }

        $('#3dContainer').on('mousedown'     , mousedown);
        $('#3dContainer').on('mouseup'       , mouseup);
        $('#3dContainer').on('mouseout'      , mouseup);
        $('#3dContainer').on('mousemove'     , mousemove);
        $('#3dContainer').on('mousewheel'    , mousewheel);
        $('#3dContainer').on('DOMMouseScroll', mousewheel);
    }
    else {
        function touchstart( event ) {
            event.preventDefault();
            initFinger1 = new THREE.Vector2( event.touches[0].pageX, event.touches[0].pageY );
            
            if (event.touches.length > 1) {
                initFinger2  = new THREE.Vector2(event.touches[1].pageX, event.touches[1].pageY);
                initDistance = initFinger2.distanceTo(initFinger1);
            }
        }

        function touchmove(event) {
            event.preventDefault();
            if (event.touches.length === 1) {
                herramientaSeleccionada = "rotar";

                angX = (event.touches[0].pageX - initFinger1.x) / config3D.visor.ancho * 45 * 0.08;
                angY = (event.touches[0].pageY - initFinger1.y) / config3D.visor.alto  * 45 * 0.08;
                    
                moveCamera(angX, angY);
            } else if (event.touches.length > 1) {
                var finger1   = new THREE.Vector2(event.touches[0].pageX, event.touches[0].pageY);
                var finger2   = new THREE.Vector2(event.touches[1].pageX, event.touches[1].pageY);
                var x         = (finger1.x - initFinger1.x) / config3D.visor.ancho * 45 * 0.04 * objetoSphere.position.distanceTo(camera.position);
                var y         = (finger1.y - initFinger1.y) / config3D.visor.ancho * 45 * 0.04 * objetoSphere.position.distanceTo(camera.position);
                var distance  = finger2.distanceTo(finger1) - initDistance;

                if (Math.abs(distance) > 10)
                    moveCameraDo(-distance / 200);

                herramientaSeleccionada = "mover";
                moveCamera(x, y);

                initFinger2.y = event.touches[1].pageY;
                initFinger2.x = event.touches[1].pageX;
            }
            initFinger1.y = event.touches[0].pageY;
            initFinger1.x = event.touches[0].pageX;
        }

        function touchend(event) {
            initFinger1  = null;
            initFinger2  = null;
            initDistance = null;
        }

        document.getElementById('3dContainer').addEventListener('touchend'  , touchend  , false);
        document.getElementById('3dContainer').addEventListener('touchstart', touchstart, false);
        document.getElementById('3dContainer').addEventListener('touchmove' , touchmove , false);
    }

    //%%%%%%%%%%%%%%%%% - moveCamera - %%%%%%%%%%%%%%%%%%%%%%///////////////////////////////////////////////////////////////////////////////////////////////
    function moveCamera(x, y) {
        switch (herramientaSeleccionada) {
            case "mover":
                camera.translateY( y);
                camera.translateX(-x);
                break;
            case "rotar":
                objetoSphere.rotation.x -= y;
                objetoSphere.rotation.y += x;
                break;
        }
    }

    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    function moveCameraDo(ratio) {
        var distance = camera.position.distanceTo(objetoSphere.position);
        var min      = 22;
        var max      = 150;

        if ( (distance > min && distance < max) ||
             (distance < min && ratio > 0)      ||
             (distance > max && ratio < 0) ) {
            camera.translateZ(ratio);
        }
    }

    return {
        doZoomIn: function() {
            moveCameraDo(-2);
        },

        doZoomOut: function() {
            moveCameraDo(2);
        },

        resetView: function() {
            objetoSphere.position.set( basePosition.position.x, basePosition.position.y, basePosition.position.z);
            objetoSphere.rotation.set( basePosition.rotation.x, basePosition.rotation.y, basePosition.rotation.z);
            // var objetoReferencia = scene.getObjectByName('referencia', true) || false;
            camera.position.set( baseCamera.position.x, baseCamera.position.y, baseCamera.position.z ); // hay que buscar un ratio bueno "!!!!!!"
            camera.lookAt( objetoSphere.position );
            camera.fov = 45;
            camera.updateProjectionMatrix();
        }
    }
}

function parseScene(jsscene, jsmeshes, jsanimation) {
    // RENDERER > BACKGROUND 
    var bg = jsscene.background || 0xFFFFFF;
    renderer.setClearColor( bg );
    // SCENE > ambient
    var a = jsscene.ambient || { r:0, g:0, b:0 };
    var ambient = new THREE.AmbientLight( new THREE.Color( a.r, a.g, a.b ) );
    scene.add( ambient );
    // MESH <> PARSE
    for ( var mi in jsmeshes ){
        var g = new THREE.Geometry();
        var vs = jsmeshes[ mi ].vertices || false;
        var us = jsmeshes[ mi ].uv1 || false;
        var ns = jsmeshes[ mi ].tris || false;
        // VERTICES
        if ( !!vs ){
            for( var i = 0, n = vs.length; i < n; i += 3 ){
                g.vertices.push( new THREE.Vector3( vs[i], vs[i+1], vs[i+2] ) );
            }
        }
        // FACES ( with UVS )
        if ( !!ns && !!us ){
            for( var i = 0, n = ns.length; i < n; i += 3 ){
                g.faces.push( new THREE.Face3( ns[i], ns[i+2], ns[i+1] ) );
                var vI1 = new THREE.Vector2( us[ ns[i]*2 ], us[ ns[i]*2+1 ] );
                var vI2 = new THREE.Vector2( us[ ns[i+2]*2 ], us[ ns[i+2]*2+1 ] );
                var vI3 = new THREE.Vector2( us[ ns[i+1]*2 ], us[ ns[i+1]*2+1 ] );
                g.faceVertexUvs[0].push( [vI1, vI2, vI3] );
            }
        }
        g.computeFaceNormals();
        g.normalsNeedUpdate = true;
        jsmeshes[ mi ] = g; // todo ! esto no lo tengo muy claro...
    }
    // MATERIAL <> PARSE
    for ( var ms in jsscene.materials ){
        var md = jsscene.materials[ ms ];
        var cd = md.color;
        var c = new THREE.Color( cd.r, cd.g, cd.b );
        var m;
        if ( !!jsscene.materials[ms].colorTexture ){
            var path2Texture = nrd.config.url +"/files/textures/"+ jsscene.textures[ jsscene.materials[ms].colorTexture ].file ;
            var mytex = THREE.ImageUtils.loadTexture( path2Texture, THREE.UVMapping );
            m = new THREE.MeshPhongMaterial({ 
                color: c 
            });
            m.map = mytex;
        }
        else {
            m = new THREE.MeshPhongMaterial({ 
                color: c 
            });
        }
        m.shininess = md.shininess;
        m.opacity = jsscene.materials[ms].color.a;
        if ( m.opacity < 1 ){
            m.transparent = true;
        }
        jsscene.materials[ ms ] = m;
    }
    // TRANSFORM <> PARSE
    for (var i = 0, len = jsscene.transforms.length; i < len; i++) {
        var td = jsscene.transforms[i];
        var t;
        if (!!td.mesh){
            t = new THREE.Mesh();
        }
        else if (!!td.camera) {
            t = new THREE.PerspectiveCamera();
        }
        else {
            t = new THREE.Object3D();
        }
        t.j3dData = td;
        t.name = td.name;
        var p  = td.position;
        t.position.set(p[0], p[1], p[2]);
        if(jsscene.quaternions) {
            var r = td.rotation;
            t.quaternion.set(r[0], r[1], r[2], r[3]);
        }
        else {
            console.log( 'Euler rotations are not supported, use quaternions!' );
        }
        if(t.scale) {
            var s = td.scale || false;
            if (!!s){
                t.scale.set(s[0], s[1], s[2]);
            }
        }
        if(td.mesh) {   
            t.geometry = jsmeshes[td.meshId];
        }
        if(td.renderer) {
            t.material = jsscene.materials[td.renderer];
        }
        if (td.camera) {
            var c    = jsscene.cameras[td.camera];
            t.fov    = c.fov;
            t.near   = c.near;
            t.far    = c.far;
            t.aspect = config3D.visor.ratioXY;
            t.updateProjectionMatrix();
        }
        jsscene.transforms[i] = t;
    }
    for (var i = 0; i < jsscene.transforms.length; i++) {
        var t = jsscene.transforms[i];
        // scene.add(t);
        if ( t.j3dData.parent != null ) {
            var padre = findByUID( jsscene.transforms, t.j3dData.parent );
            padre.add(t);
        }
        else {
            scene.add(t);
        }
    }
    loadCapas();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function loadCapas() {
    var menuContainer = '<div id="containerMenuContainer"><div id="menuContainer"></div></div>';
    $("#content").append( menuContainer );
    $( "#containerMenuContainer" ).hide();
    controles = new setControls();
    var req = $.getJSON( nrd.config.url + "/capas.json", {}, function( json ){
        tipoInteraccion = json.tipoInteraccion || false;
        if ( isMobile ) {
            tipoInteraccion = "resaltar";
        }
        if ( !tipoInteraccion ){
            return;
        }
        var icono;
        switch ( tipoInteraccion ){
            case "ocultar":
                icono = "visualizar";
                break;
            case "resaltar":
            default:
                icono = "bombilla";
                break;
        }
        addButton( "verMenu", "right", icono, "menu" );
        bindClick2Button( "verMenu", "verMenu()" );
        for ( var i = 0; i < json.capas.length; i++ ){
            
            if ( tipoInteraccion == "ocultar" ){
                var boton = '<div id="botonMenu_'+i+'" class="botonMenu" uid="'+ json.capas[i].uid +'"><p><a class="ojo"></a>'+ json.capas[i].nombre +'</p></div>';
            }
            else {
                var boton = '<div id="botonMenu_'+i+'" class="botonMenu" uid="'+ json.capas[i].uid +'"><p><a class="luz"></a>'+ json.capas[i].nombre +'</p></div>';   
            }
            $("#menuContainer").append( boton );
            // EVENTOS DE CONTROLES
            if( !isMobile ){
                if ( tipoInteraccion == "resaltar" ){
                    $("#botonMenu_"+i).on( 'mouseover', {o: scene.getObjectByName(json.capas[i].uid)}, function( event ){
                        if ( !!arrayMenuSeleccinado[ event.data.i ] ){
                            return;
                        }
                        $(this).addClass( "selected" );
                        selectMaps( event.data.o, true );
                    });
                    $("#botonMenu_"+i).on( 'mouseout', {o: scene.getObjectByName(json.capas[i].uid), i:i }, function( event ){
                        if ( !!arrayMenuSeleccinado[ event.data.i ] ){
                            return;
                        }
                        $(this).removeClass( "selected" );
                        selectMaps( event.data.o, false );
                    });
                }
                $("#botonMenu_"+i).on( 'click', {o: scene.getObjectByName(json.capas[i].uid), i:i }, function( event ){
                    if ( !!arrayMenuSeleccinado[ event.data.i ] ){ // ya estaba...
                        var _obj = arrayMenuSeleccinado[ event.data.i ];
                        if ( tipoInteraccion == "ocultar" ){
                            $("#"+_obj.id).addClass("clicked");
                            $("#"+_obj.id).addClass("selected");
                        }
                        else {
                            $("#"+_obj.id).removeClass("clicked");
                            $("#"+_obj.id).removeClass("selected");
                        }
                        selectMaps( _obj.obj, false );
                        arrayMenuSeleccinado[ event.data.i ] = false;
                    }
                    else { // otro nuevo...
                        var itemMenuSeleccionado = {};
                        itemMenuSeleccionado.id = "botonMenu_"+event.data.i;
                        itemMenuSeleccionado.obj = event.data.o;
                        if ( tipoInteraccion == "ocultar" ){
                            $( "#"+itemMenuSeleccionado.id).removeClass("clicked");
                            $( "#"+itemMenuSeleccionado.id ).removeClass("selected");
                        }
                        else {
                            $( "#"+itemMenuSeleccionado.id).addClass("clicked");
                            $( "#"+itemMenuSeleccionado.id).addClass("selected");
                        }
                        selectMaps( itemMenuSeleccionado.obj, true );
                        arrayMenuSeleccinado[ event.data.i ] = itemMenuSeleccionado;
                    }
                });
            }
            else {
                $("#botonMenu_"+i).on( 'touchstart', {o: scene.getObjectByName(json.capas[i].uid)}, function( event ){
                    if ( !!arrayMenuSeleccinado[ event.data.i ] ){
                        return;
                    }
                    $(this).addClass( "selected" );
                    inTouchObject = event.data.o;
                    selectMaps( event.data.o, true );
                });
                $("#botonMenu_"+i).on( 'touchout', {o: scene.getObjectByName(json.capas[i].uid)}, function( event ){
                    if ( !!arrayMenuSeleccinado[ event.data.i ] ){
                        return;
                    }
                    $(".selected").removeClass( "selected" );
                    selectMaps( inTouchObject, false );
                });
                $(document).on( 'touchend', function( event ){
                    $(".botonMenu.selected").removeClass( "selected" );
                    selectMaps( inTouchObject, false );
                });
            }
        }
        interCapas = json.capas;
        if ( tipoInteraccion == "ocultar" ){
            $(".botonMenu").addClass("selected");
            $(".botonMenu").addClass("clicked");
        }
    });
    $("#loaderWEBGL").remove();
    if ( !isMobile ){
        addButton( "zoomIn", "left", "zoomin" ,"zoom +");
        addButton( "zoomOut", "left", "zoomout" ,"zoom -");
        bindClick2Button( "zoomIn", "zoomIn()" );
        bindClick2Button( "zoomOut", "zoomOut()" );
        addButton( "rotar", "left", "rotar", "rotate" );
        addButton( "mover", "left", "mover", "move" );
        bindClick2Button( "rotar", 'selectTool( "rotar" )' );
        bindClick2Button( "mover", 'selectTool( "mover" )' );
        $("#mover").addClass( "jstool" );
        $("#rotar").addClass( "jstool" );
    }
    addButton( "reiniciar", "left", "reiniciar", "reload" );
    bindClick2Button( "reiniciar", "resetView()" );
}

function doRender() {
    renderer.render( scene, camera );
    requestAnimationFrame( doRender );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function findByUID(where, which) {
    for (var i = 0; i < where.length; i++) {
        if (where[i].j3dData.uid == which) {
            return where[i];  
        } 
    }
    return false;
}

// function searchInScene(uid) {
//     return new getObjects(scene, uid);
// }

function getObjects(obj, val) {
    var key = "j3dData";
    if ( !!obj[key] && obj[key].uid == val ){
        gottenObject = obj;
        return;
    }
    if ( obj.children.length > 0 ){     
        for ( var i = 0; i < obj.children.length; i++ ){ 
            getObjects( obj.children[i], val );         
        }
        return obj;
    }
    else {
        return obj;
    }
}

///////////////////////////////////////////////////////////////////////////////////////////////////////
function selectTool(tool) {
    if ( !herramientaSeleccionada ){
        herramientaSeleccionada = tool;
        $("#"+tool).addClass("selected");
    }
    else {
        $("#"+herramientaSeleccionada).removeClass("selected");
        if ( herramientaSeleccionada == tool ){
            herramientaSeleccionada = false;
        }
        else {
            $("#"+tool).addClass("selected");
            herramientaSeleccionada = tool;
        }
    }
}

function verMenu() {
    if ( showMenu ){
        showMenu = false;
        $( "#verMenu" ).removeClass( "selected" );
        $( "#3dContainer" ).removeClass( "aIzquierda" );
        $( "#containerMenuContainer" ).hide();
    }
    else {
        showMenu = true;
        $( "#verMenu" ).addClass( "selected" );
        $( "#3dContainer" ).addClass( "aIzquierda" );
        $( "#containerMenuContainer" ).show();
    }
}
/// #NRD functions /////////////////////////////////////////////////////////

function zoomIn() {
    controles.doZoomIn();
}

function zoomOut() {
    controles.doZoomOut();  
}

function resetView() {
    if ( !!showMenu ){
        verMenu();
    }
    selectTool( false );
    controles.resetView();
}

function addToSelectable(o, uid, text) {
    if (o.j3dData && o.j3dData.uid == uid) {
        var newSel = {
            object: o,
            id: uid,
            text: text
        };  
        objetoCapa.push(newSel);    
        botones(newSel);
    }
}

function selectMaps(obj, select) {
    switch (tipoInteraccion) {
        case "resaltar":
            if (select) {
                resaltarObjeto(obj);
            }
            else {
                mostrarObjeto(obj);
            }
            break;
        case "ocultar":
            if (select) {
                ocultarObjeto(obj);
            }
            else {
                mostrarObjeto(obj);
            }
            break;
    }
    if (obj.children && obj.children.length > 0) {      
        for (var i = 0; i < obj.children.length; i++) {
            selectMaps(obj.children[i], select);    
        }
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function mostrarObjeto(obj) {
    obj.material = materialesGuardados[obj.uuid];   
}

function resaltarObjeto(obj) {
    if (!materialesGuardados[obj.uuid]) {
        materialesGuardados[obj.uuid] = obj.material;
    }
    var newMaterial = jQuery.extend({}, obj.material);
    var materialSettings = { 
        color: new THREE.Color(0xEE44CC)
    }
    if (!!obj.material) {
        materialSettings.map = obj.material.map;
    }
    obj.material = new THREE.MeshLambertMaterial(materialSettings);
}

function ocultarObjeto(obj) {
    materialesGuardados[obj.uuid] = obj.material;
    var newMaterial               = jQuery.extend({}, obj.material);
    newMaterial.opacity           = 0;
    newMaterial.transparent       = true;
    obj.material                  = obj.newMaterial;
}